export const menu = (
  selection,
  { data, value = 'Select Artist', onChange }
) => {
  const artists = [
    ...new Set(data.map((d) => d.Artist)),
  ];

  const selectElement = selection
    .selectAll('select')
    .data([null])
    .join('select')
    .on('change', (event) => {
      const selectedArtist = event.target.value;
      onChange(selectedArtist);
      const filteredData = data.filter(
        (d) => d.Artist === selectedArtist
      );
      displayArtworksByYear(filteredData);
    });

  selectElement
    .selectAll('option')
    .data(['Select Artist', ...artists])
    .join('option')
    .attr('value', (d) => d)
    .property('selected', (d) => d === value)
    .text((d) => d);

  const displayArtworksByYear = (
    filteredData
  ) => {
    const artworkCountByYear = d3.rollup(
      filteredData,
      (v) => v.length,
      (d) => d.Date.split('-')[0]
    );
    const artworkCountByYearArray = Array.from(
      artworkCountByYear,
      ([key, value]) => ({
        year: key,
        count: value,
      })
    );

    const leftPadding = 100; // set desired left padding value
    const bottomPadding = 70; // set desired bottom padding value
    const rightPadding = 50; // set desired right padding value

    const svg = selection
      .selectAll('svg')
      .data([null])
      .join('svg')
      .attr('width', window.innerWidth - 40)
      .attr('height', window.innerHeight - 100)
      .attr('transform', 'translate(0,5)')
      .style('overflow', 'auto');

    // Remove all existing rect elements
    svg.selectAll('g').remove();
    var margin = {
      top: 20,
      right: 20,
      bottom: 20,
      left: 20,
    };

    var buttonGroup = svg.append('g'); // Move the group to (50, 50)
    var chartGroup = svg
      .append('g')
      .style('overflow', 'auto')

      .attr('transform', 'translate(0,10)');
    // Move the group to (50, 50)

    // Create a button
    var button = buttonGroup
      .append('rect')
      .attr('x', 10)
      .attr('y', 0)
      .attr('width', 120)
      .attr('height', 40)
      .style('fill', 'black');

    var text = buttonGroup
      .append('text')
      .attr('x', 25)
      .attr('y', 23)
      .attr('fill', 'white')
      .style('font-family', 'sans-serif')
      .text('View Gallery');

    button.on('click', function () {
      // Get the thumbnail URLs from the data
      const thumbnailUrls = filteredData.map(
        (artwork) => artwork.ThumbnailURL
      );
      var galleryContainer = svg
        .append('div')
        .attr('id', 'gallery-container');

      // // Create a container for the gallery view
      // var galleryContainer = d3.select(
      //   '#gallery-container'
      // );

      // Bind the thumbnail URLs to image elements
      var images = galleryContainer
        .selectAll('img')
        .data(thumbnailUrls)
        .enter()
        .append('img')
        .attr('src', function (d) {
          return d;
        }) // Set the image source to the URL
        .attr('width', '150') // Set the image width
        .attr('height', '150') // Set the image height
        .style('margin', '5px'); // Add some margin between images

      // Open a new window with the gallery view
      var popupWindow = window.open(
        '',
        'Gallery View',
        'width=800,height=600'
      );
      popupWindow.document.write(
        galleryContainer.html()
      );
    });

    text.on('click', function () {
      // Get the thumbnail URLs from the data
      const thumbnailUrls = filteredData.map(
        (artwork) => artwork.ThumbnailURL
      );
      var galleryContainer = svg
        .append('div')
        .attr('id', 'gallery-container');

      // // Create a container for the gallery view
      // var galleryContainer = d3.select(
      //   '#gallery-container'
      // );

      // Bind the thumbnail URLs to image elements
      var images = galleryContainer
        .selectAll('img')
        .data(thumbnailUrls)
        .enter()
        .append('img')
        .attr('src', function (d) {
          return d;
        }) // Set the image source to the URL
        .attr('width', '150') // Set the image width
        .attr('height', '150') // Set the image height
        .style('margin', '5px'); // Add some margin between images

      // Open a new window with the gallery view
      var popupWindow = window.open(
        '',
        'Gallery View',
        'width=800,height=600'
      );
      popupWindow.document.write(
        galleryContainer.html()
      );
    });

    // Create a button
    var button2 = buttonGroup
      .append('rect')
      .attr('x', 140)
      .attr('y', 0)
      .attr('width', 145)
      .attr('height', 40)
      .style('fill', 'black');

    var text2 = buttonGroup
      .append('text')
      .attr('x', 155)
      .attr('y', 23)
      .attr('fill', 'white')
      .style('font-family', 'sans-serif')
      .text('View Histogram');

    button2.on('click', function () {
      const y = d3
        .scaleLinear()
        .domain([
          0,
          d3.max(
            artworkCountByYearArray,
            (d) => d.count
          ),
        ])
        .nice()
        .range([
          window.innerHeight -
            100 -
            bottomPadding,
          40,
        ]);

      const barWidth = Math.min(
        20,
        (window.innerWidth -
          leftPadding -
          rightPadding -
          20) /
          artworkCountByYearArray.length
      );

      const x = d3
        .scaleBand()
        .domain(
          artworkCountByYearArray
            .map((d) => d.year)
            .sort()
        )
        .range([
          leftPadding,
          window.innerWidth - rightPadding - 20,
        ])
        .padding(0.2);
      const bars = chartGroup
        .selectAll('.bar')
        .data(artworkCountByYearArray)
        .enter()
        .append('rect')
        .attr('class', 'bar')
        .attr('x', (d) => x(d.year))
        .attr('y', (d) => y(d.count))
        .attr('width', x.bandwidth())
        .attr(
          'height',
          (d) =>
            window.innerHeight -
            100 -
            bottomPadding -
            y(d.count)
        )
        .attr('fill', 'steelblue')
        .attr('rx', 5)
        .attr('ry', 5)
        .on('mouseover', function (event, d) {
          const [xPos, yPos] = d3.pointer(event);
          d3.select(this)
            .style('fill', 'orange')
            .style('cursor', 'pointer');
          tooltip
            .style('display', 'block')
            .style('left', `${xPos}px`)
            .style('top', `${yPos + 100}px`)
            .html(
              `Year: ${d.year}<br>Count: ${d.count}`
            );
        })
        .on('mouseout', function (event, d) {
          d3.select(this)
            .style('fill', 'steelblue')
            .style('cursor', 'default');
          tooltip.style('display', 'none');
        });

      const xAxis = d3
        .axisBottom(x)
        .ticks(10)
        .tickFormat((d) => d);

      // Create the tooltip
      const tooltip = d3
        .select('body')
        .append('div')
        .attr('class', 'tooltip')
        .style('position', 'absolute')
        .style('display', 'none')
        .style('background-color', 'white')
        .style('border', 'solid 1px black')
        .style('border-radius', '5px') // Set border radius
        .style('padding', '10px')
        .style('font-size', '12px')
        .style('font-family', 'sans-serif')
        .style('pointer-events', 'none');

      const yAxis = d3.axisLeft(y); // set the tick format to display integers only
      // set the number of ticks to the maximum y value in the data;

      chartGroup
        .selectAll('g.axis-x')
        .data([null])
        .join('g')
        .classed('axis-x', true)
        .call(xAxis)
        .attr(
          'transform',
          `translate(0, ${
            window.innerHeight -
            100 -
            bottomPadding
          })`
        );

      chartGroup
        .selectAll('text.axis-x-label')
        .data([null])
        .join('text')
        .classed('axis-x-label', true)
        .attr('x', 100)
        .attr(
          'y',
          window.innerHeight - 80 - bottomPadding
        )
        .attr('text-anchor', 'middle')
        .style('font-family', 'sans-serif')
        .text('Year');

      chartGroup
        .selectAll('g.axis-y')
        .data([null])
        .join('g')
        .classed('axis-y', true)
        .call(yAxis)
        .attr(
          'transform',
          `translate(${leftPadding}, 0)`
        );

      chartGroup
        .selectAll('text.axis-y-label')
        .data([null])
        .join('text')
        .classed('axis-y-label', true)
        .attr(
          'x',
          -((window.innerHeight - 50) / 2)
        )
        .attr('y', 60)
        .attr('text-anchor', 'middle')
        .style('font-family', 'sans-serif')
        .attr('transform', 'rotate(-90)')
        .text('Number of Artworks');
    });

    text2.on('click', function () {
      const y = d3
        .scaleLinear()
        .domain([
          0,
          d3.max(
            artworkCountByYearArray,
            (d) => d.count
          ),
        ])
        .nice()
        .range([
          window.innerHeight -
            100 -
            bottomPadding,
          40,
        ]);

      const barWidth = Math.min(
        20,
        (window.innerWidth -
          leftPadding -
          rightPadding -
          20) /
          artworkCountByYearArray.length
      );

      const x = d3
        .scaleBand()
        .domain(
          artworkCountByYearArray
            .map((d) => d.year)
            .sort()
        )
        .range([
          leftPadding,
          window.innerWidth - rightPadding - 20,
        ])
        .padding(0.2);
      const bars = chartGroup
        .selectAll('.bar')
        .data(artworkCountByYearArray)
        .enter()
        .append('rect')
        .attr('class', 'bar')
        .attr('x', (d) => x(d.year))
        .attr('y', (d) => y(d.count))
        .attr('width', x.bandwidth())
        .attr(
          'height',
          (d) =>
            window.innerHeight -
            100 -
            bottomPadding -
            y(d.count)
        )
        .attr('fill', 'steelblue')
        .attr('rx', 5)
        .attr('ry', 5)
        .on('mouseover', function (event, d) {
          const [xPos, yPos] = d3.pointer(event);
          d3.select(this)
            .style('fill', 'orange')
            .style('cursor', 'pointer');
          tooltip
            .style('display', 'block')
            .style('left', `${xPos}px`)
            .style('top', `${yPos + 100}px`)
            .html(
              `Year: ${d.year}<br>Count: ${d.count}`
            );
        })
        .on('mouseout', function (event, d) {
          d3.select(this)
            .style('fill', 'steelblue')
            .style('cursor', 'default');
          tooltip.style('display', 'none');
        });

      const xAxis = d3
        .axisBottom(x)
        .ticks(10)
        .tickFormat((d) => d);

      // Create the tooltip
      const tooltip = d3
        .select('body')
        .append('div')
        .attr('class', 'tooltip')
        .style('position', 'absolute')
        .style('display', 'none')
        .style('background-color', 'white')
        .style('border', 'solid 1px black')
        .style('border-radius', '5px') // Set border radius
        .style('padding', '10px')
        .style('font-size', '12px')
        .style('font-family', 'sans-serif')
        .style('pointer-events', 'none');

      const yAxis = d3.axisLeft(y); // set the tick format to display integers only
      // set the number of ticks to the maximum y value in the data;

      chartGroup
        .selectAll('g.axis-x')
        .data([null])
        .join('g')
        .classed('axis-x', true)
        .call(xAxis)
        .attr(
          'transform',
          `translate(0, ${
            window.innerHeight -
            100 -
            bottomPadding
          })`
        );

      chartGroup
        .selectAll('text.axis-x-label')
        .data([null])
        .join('text')
        .classed('axis-x-label', true)
        .attr('x', 100)
        .attr(
          'y',
          window.innerHeight - 80 - bottomPadding
        )
        .attr('text-anchor', 'middle')
        .style('font-family', 'sans-serif')
        .text('Year');

      chartGroup
        .selectAll('g.axis-y')
        .data([null])
        .join('g')
        .classed('axis-y', true)
        .call(yAxis)
        .attr(
          'transform',
          `translate(${leftPadding}, 0)`
        );

      chartGroup
        .selectAll('text.axis-y-label')
        .data([null])
        .join('text')
        .classed('axis-y-label', true)
        .attr(
          'x',
          -((window.innerHeight - 50) / 2)
        )
        .attr('y', 60)
        .attr('text-anchor', 'middle')
        .style('font-family', 'sans-serif')
        .attr('transform', 'rotate(-90)')
        .text('Number of Artworks');
    });
  };
};
